@@ -85,7 +85,7 @@ char *DirManager::directory(const char *subdir, const QString &desc)
 			"It will be used to store %2.\nShould I create it now?")
 			.arg(dir).arg(desc);
 
-		if(KMessageBox::questionYesNo(0,message,i18n("aRts Folder Missing"))
+		if(KMessageBox::questionYesNo(0,message,i18n("aRts Folder Missing"),i18n("Create Folder"),i18n("Do Not Create"))
 			== KMessageBox::Yes)
 		{
 			KStandardDirs::makeDir(dir);
