@@ -24,11 +24,13 @@
 #if HAVE_MUSICBRAINZ
 
 #include <kapplication.h>
+#include <kresolver.h>
 #include <kprotocolmanager.h>
 #include <kurl.h>
 #include <kdebug.h>
 
 #include <qmutex.h>
+#include <qregexp.h>
 #include <qevent.h>
 #include <qobject.h>
 #include <qfile.h>
@@ -40,7 +42,11 @@ class KTRMLookup;
 
 extern "C"
 {
+#if HAVE_MUSICBRAINZ >= 4
+    static void TRMNotifyCallback(tunepimp_t pimp, void *data, TPCallbackEnum type, int fileId, TPFileStatus status);
+#else
     static void TRMNotifyCallback(tunepimp_t pimp, void *data, TPCallbackEnum type, int fileId);
+#endif
 }
 
 /**
@@ -59,12 +65,16 @@ public:
         return &handler;
     }
 
-    int KTRMRequestHandler::startLookup(KTRMLookup *lookup)
+    int startLookup(KTRMLookup *lookup)
     {
         int id;
 
         if(!m_fileMap.contains(lookup->file())) {
+#if HAVE_MUSICBRAINZ >= 4
+            id = tp_AddFile(m_pimp, QFile::encodeName(lookup->file()), 0);
+#else
             id = tp_AddFile(m_pimp, QFile::encodeName(lookup->file()));
+#endif
             m_fileMap.insert(lookup->file(), id);
         }
         else {
@@ -118,12 +128,59 @@ protected:
         tp_SetAutoSaveThreshold(m_pimp, -1);
         tp_SetMoveFiles(m_pimp, false);
         tp_SetRenameFiles(m_pimp, false);
+#if HAVE_MUSICBRAINZ >= 4
+        tp_SetFileNameEncoding(m_pimp, "UTF-8");
+#else
         tp_SetUseUTF8(m_pimp, true);
+#endif
         tp_SetNotifyCallback(m_pimp, TRMNotifyCallback, 0);
 
+        // Re-read proxy config.
+        KProtocolManager::reparseConfiguration();
+
         if(KProtocolManager::useProxy()) {
-            KURL proxy = KProtocolManager::proxyFor("http");
-            tp_SetProxy(m_pimp, proxy.host().latin1(), short(proxy.port()));
+            // split code copied from kcm_kio.
+            QString noProxiesFor = KProtocolManager::noProxyFor();
+            QStringList noProxies = QStringList::split(QRegExp("[',''\t'' ']"), noProxiesFor);
+            bool useProxy = true;
+
+            // Host that libtunepimp will contact.
+            QString tunepimpHost = "www.musicbrainz.org";
+            QString tunepimpHostWithPort = "www.musicbrainz.org:80";
+
+            // Check what hosts are allowed to proceed without being proxied,
+            // or is using reversed proxy, what hosts must be proxied.
+            for(QStringList::ConstIterator it = noProxies.constBegin(); it != noProxies.constEnd(); ++it) {
+                QString normalizedHost = KNetwork::KResolver::normalizeDomain(*it);
+
+                if(normalizedHost == tunepimpHost ||
+                   tunepimpHost.endsWith("." + normalizedHost))
+                {
+                    useProxy = false;
+                    break;
+                }
+
+                // KDE's proxy mechanism also supports exempting a specific
+                // host/port combo, check that also.
+                if(normalizedHost == tunepimpHostWithPort ||
+                   tunepimpHostWithPort.endsWith("." + normalizedHost))
+                {
+                    useProxy = false;
+                    break;
+                }
+            }
+
+            // KDE supports a reverse proxy mechanism.  Uh, yay.
+            if(KProtocolManager::useReverseProxy())
+                useProxy = !useProxy;
+
+            if(useProxy) {
+                KURL proxy = KProtocolManager::proxyFor("http");
+                QString proxyHost = proxy.host();
+
+                kdDebug(65432) << "Using proxy server " << proxyHost << " for www.musicbrainz.org.\n";
+                tp_SetProxy(m_pimp, proxyHost.latin1(), short(proxy.port()));
+            }
         }
     }
 
@@ -241,14 +298,19 @@ protected:
 /**
  * Callback fuction for TunePimp lookup events.
  */
-
-static void TRMNotifyCallback(tunepimp_t pimp, void *, TPCallbackEnum type, int fileId)
+#if HAVE_MUSICBRAINZ >= 4
+static void TRMNotifyCallback(tunepimp_t pimp, void *data, TPCallbackEnum type, int fileId, TPFileStatus status)
+#else
+static void TRMNotifyCallback(tunepimp_t pimp, void *data, TPCallbackEnum type, int fileId)
+#endif
 {
     if(type != tpFileChanged)
         return;
 
     track_t track = tp_GetTrack(pimp, fileId);
+#if HAVE_MUSICBRAINZ < 4
     TPFileStatus status = tr_GetStatus(track);
+#endif
 
     switch(status) {
     case eRecognized:
@@ -458,10 +520,16 @@ void KTRMLookup::collision()
                 KTRMResult result;
 
                 result.d->title = QString::fromUtf8(tracks[i]->name);
+#if HAVE_MUSICBRAINZ >= 4
+                result.d->artist = QString::fromUtf8(tracks[i]->artist.name);
+                result.d->album = QString::fromUtf8(tracks[i]->album.name);
+                result.d->year = tracks[i]->album.releaseYear;
+#else 
                 result.d->artist = QString::fromUtf8(tracks[i]->artist->name);
                 result.d->album = QString::fromUtf8(tracks[i]->album->name);
-                result.d->track = tracks[i]->trackNum;
                 result.d->year = tracks[i]->album->releaseYear;
+#endif
+                result.d->track = tracks[i]->trackNum;
                 result.d->relevance = tracks[i]->relevance;
 
                 d->results.append(result);
@@ -505,3 +573,5 @@ void KTRMLookup::finished()
 }
 
 #endif
+
+// vim: set et ts=8 sw=4:
