@@ -24,6 +24,7 @@
 #include <qvaluevector.h>
 #include <qfileinfo.h>
 
+#include "covermanager.h"
 #include "stringhash.h"
 #include "playlistsearch.h"
 #include "tagguesser.h"
@@ -319,6 +320,8 @@ public:
 
     void read(QDataStream &s);
 
+    static void setShuttingDown() { m_shuttingDown = true; }
+
 public slots:
     /**
      * Remove the currently selected items from the playlist and disk.
@@ -382,6 +385,11 @@ public slots:
      */
     void slotRemoveCover();
 
+    /**
+     * Shows the cover manager GUI dialog
+     */
+    void slotShowCoverManager();
+
     /**
      * Reload the playlist contents from the m3u file.
      */
@@ -410,14 +418,16 @@ protected:
     // the following are all reimplemented from base classes
 
     virtual bool eventFilter(QObject *watched, QEvent *e);
+    virtual void keyPressEvent(QKeyEvent *e);
     virtual QDragObject *dragObject(QWidget *parent);
     virtual QDragObject *dragObject() { return dragObject(this); }
     virtual bool canDecode(QMimeSource *s);
     virtual void decode(QMimeSource *s, PlaylistItem *item = 0);
     virtual void contentsDropEvent(QDropEvent *e);
     virtual void contentsMouseDoubleClickEvent(QMouseEvent *e);
+    virtual void contentsDragEnterEvent(QDragEnterEvent *e);
     virtual void showEvent(QShowEvent *e);
-    virtual bool acceptDrag(QDropEvent *e) const { return KURLDrag::canDecode(e); }
+    virtual bool acceptDrag(QDropEvent *e) const;
     virtual void viewportPaintEvent(QPaintEvent *pe);
     virtual void viewportResizeEvent(QResizeEvent *re);
 
@@ -449,7 +459,7 @@ protected:
      * Forwards the call to the parent to enable or disable automatic deletion
      * of tree view playlists.  Used by CollectionListItem.
      */
-    void setCanDeletePlaylist(bool canDelete);
+    void setDynamicListsFrozen(bool frozen);
 
     template <class ItemType, class SiblingType>
     ItemType *createItem(SiblingType *sibling, ItemType *after = 0);
@@ -534,9 +544,20 @@ private:
 
     void redisplaySearch() { setSearch(m_search); }
 
-    void refreshAlbums(const PlaylistItemList &items, const QImage &image = QImage());
+    /**
+     * Sets the cover for items to the cover identified by id.
+     */
+    void refreshAlbums(const PlaylistItemList &items, coverKey id = CoverManager::NoMatch);
+
     void refreshAlbum(const QString &artist, const QString &album);
 
+    /**
+     * Returns the number of PlaylistItems in @p items that can be assigned a
+     * cover.  Used to avoid wasting the users' time setting the cover for 20
+     * items when none are eligible.
+     */
+    unsigned int eligibleCoverItems(const PlaylistItemList &items);
+
     void updatePlaying() const;
 
     /**
@@ -626,8 +647,8 @@ private:
     bool m_disableColumnWidthUpdates;
 
     mutable int m_time;
-    mutable QValueList<PlaylistItem::Pointer> m_addTime;
-    mutable QValueList<PlaylistItem::Pointer> m_subtractTime;
+    mutable PlaylistItemList m_addTime;
+    mutable PlaylistItemList m_subtractTime;
 
     /**
      * The average minimum widths of columns to be used in balancing calculations.
@@ -636,7 +657,6 @@ private:
     QValueVector<int> m_columnFixedWidths;
     bool m_widthsDirty;
 
-    PlaylistItemList m_randomList;
     static PlaylistItemList m_history;
     PlaylistSearch m_search;
 
@@ -668,6 +688,7 @@ private:
      * call to setVisibleItems()) while random play is playing.
      */
     static bool m_visibleChanged;
+    static bool m_shuttingDown;
     static int m_leftColumn;
     static QMap<int, PlaylistItem *> m_backMenuItems;
 
@@ -676,6 +697,13 @@ private:
 
 bool processEvents();
 
+class FocusUpEvent : public QCustomEvent
+{
+public:
+    FocusUpEvent() : QCustomEvent(id) {}
+    static const int id = 999;
+};
+
 QDataStream &operator<<(QDataStream &s, const Playlist &p);
 QDataStream &operator>>(QDataStream &s, Playlist &p);
 
@@ -706,9 +734,6 @@ ItemType *Playlist::createItem(const FileHandle &file, QListViewItem *after,
 	ItemType *i = after ? new ItemType(item, this, after) : new ItemType(item, this);
 	setupItem(i);
 
-        if(!m_randomList.isEmpty() && !m_visibleChanged)
-            m_randomList.append(i);
-
 	if(emitChanged)
 	    dataChanged();
 
@@ -726,8 +751,6 @@ ItemType *Playlist::createItem(SiblingType *sibling, ItemType *after)
     if(!m_members.insert(sibling->file().absFilePath()) || m_allowDuplicates) {
 	after = new ItemType(sibling->collectionItem(), this, after);
 	setupItem(after);
-	if(!m_randomList.isEmpty() && !m_visibleChanged)
-	    m_randomList.append(after);
     }
 
     m_disableColumnWidthUpdates = false;
