@@ -28,8 +28,8 @@
 
 class Playlist;
 class PlaylistItem;
-class CollectionList;
 class CollectionListItem;
+class CollectionList;
 
 typedef QValueList<PlaylistItem *> PlaylistItemList;
 
@@ -202,7 +202,11 @@ private:
 
 inline kdbgstream &operator<<(kdbgstream &s, const PlaylistItem &item)
 {
-    s << item.text(PlaylistItem::TrackColumn);
+    if(&item == 0)
+	s << "(nil)";
+    else
+	s << item.text(PlaylistItem::TrackColumn);
+
     return s;
 }
 
