@@ -5,7 +5,7 @@
  * Copyright (C) 2001 Adrian Schroeter <adrian@suse.de>
  * Copyright (C) 2003 Richard Lärkäng <richard@goteborg.utfors.se>
  * Copyright (C) 2003 Scott Wheeler <wheeler@kde.org>
- * Copyright (C) 2004, 2005 Benjamin Meyer <ben + audiocd at meyerhome dot net>
+ * Copyright (C) 2004, 2005 Benjamin Meyer <ben at meyerhome dot net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,51 +26,15 @@
 
 extern "C"
 {
-#include <cdda_interface.h>
-#include <cdda_paranoia.h>
-
-/**
- * This is in support for the Mega Hack, if cdparanoia ever is fixed, or we
- * use another ripping library we can remove this.
- **/
-#ifdef __linux__
-#ifdef __KCC
-/* KAI C++'s sys/types.h defines _I386_TYPES_H to get rid of non-ANSI things
-	 from asm/types.h, which also removes __u8,__u16, which are required
-	 by linux/cdrom.h. The only chance is to undef it again, so it gets
-	 included. (matz) */
-#undef _I386_TYPES_H
-/* And <linux/byteorder/swab.h> uses GNU C extensions without providing
-	 fallbacks for non-GNUisms. Fortunately we (or other headers) don't
-	 need the swab routines. */
-#define _LINUX_BYTEORDER_SWAB_H
-#endif
-#include <linux/version.h>
-
-#ifndef __GNUC__
-#define __GNUC__ 1
-#endif
-#undef __STRICT_ANSI__
-#include <asm/types.h>
-#include <linux/cdrom.h>
-#endif
-
-#include <sys/ioctl.h>
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-#include <sys/cdio.h>
-#endif
-
-#include <kdemacros.h>
-void paranoiaCallback(long, int);
-KDE_EXPORT int kdemain(int argc, char ** argv);
-#ifndef CDPARANOIA_STATIC
-	int FixupTOC(cdrom_drive *d, int tracks);
-#endif
+	#include <cdda_interface.h>
+	#include <cdda_paranoia.h>
+	void paranoiaCallback(long, int);
 
+	#include <kdemacros.h>
+	KDE_EXPORT int kdemain(int argc, char ** argv);
 }
 
 #include "audiocd.h"
-
 #include "plugins/audiocdencoder.h"
 
 #include <sys/stat.h>
@@ -84,11 +48,13 @@ KDE_EXPORT int kdemain(int argc, char ** argv);
 #include <qfileinfo.h>
 #include <kcmdlineargs.h>
 #include <kdebug.h>
-#include <kprotocolmanager.h>
+#include <kapplication.h>
 #include <klocale.h>
 #include <qregexp.h>
+
 // CDDB
 #include <client.h>
+#include "kcompactdisc.h"
 
 using namespace KIO;
 using namespace KCDDB;
@@ -97,104 +63,6 @@ using namespace KCDDB;
 #define DEFAULT_CD_DEVICE "/dev/cdrom"
 #define CDDB_INFORMATION "CDDB Information"
 
-int start_of_first_data_as_in_toc;
-int hack_track;
-
-/* Only do this if we have shared libcdda_cdparanoia. */
-#ifndef CDPARANOIA_STATIC
-/* Mega hack.  This function comes from libcdda_interface, and is called by
-	 it.  We need to override it, so we implement it ourself in the hope, that
-	 shared lib semantics make the calls in libcdda_interface to FixupTOC end
-	 up here, instead of it's own copy.  This usually works.
-	 You don't want to know the reason for this.  (matz) */
-int FixupTOC(cdrom_drive *d, int tracks)
-{
-	int j;
-	for (j = 0; j < tracks; j++) {
-		if (d->disc_toc[j].dwStartSector < 0)
-			d->disc_toc[j].dwStartSector = 0;
-		if (j < tracks-1
-				&& d->disc_toc[j].dwStartSector > d->disc_toc[j+1].dwStartSector)
-			d->disc_toc[j].dwStartSector = 0;
-	}
-	long last = d->disc_toc[0].dwStartSector;
-	for (j = 1; j < tracks; j++) {
-		if (d->disc_toc[j].dwStartSector < last)
-			d->disc_toc[j].dwStartSector = last;
-	}
-	start_of_first_data_as_in_toc = -1;
-	hack_track = -1;
-	if (d->ioctl_fd != -1) {
-		int ms_addr;
-#ifdef __linux__
-		struct cdrom_multisession ms_str;
-		ms_str.addr_format = CDROM_LBA;
-		if (ioctl(d->ioctl_fd, CDROMMULTISESSION, &ms_str) == -1)
-			return -1;
-		ms_addr = ms_str.addr.lba;
-#endif
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-		ms_addr = 0; /* last session */
-		if (ioctl(d->ioctl_fd, CDIOREADMSADDR, &ms_addr) == -1)
-			return -1;
-#endif
-
-		if (ms_addr > 100) {
-			for (j = tracks-1; j >= 0; j--)
-				if (j > 0 && !IS_AUDIO(d,j) && IS_AUDIO(d,j-1)) {
-					if (d->disc_toc[j].dwStartSector > ms_addr - 11400) {
-						/* The next two code lines are the purpose of duplicating this
-						 * function, all others are an exact copy of paranoias FixupTOC().
-						 * The gory details: CD-Extra consist of N audio-tracks in the
-						 * first session and one data-track in the next session.  This
-						 * means, the first sector of the data track is not right behind
-						 * the last sector of the last audio track, so all length
-						 * calculation for that last audio track would be wrong.  For this
-						 * the start sector of the data track is adjusted (we don't need
-						 * the real start sector, as we don't rip that track anyway), so
-						 * that the last audio track end in the first session.  All well
-						 * and good so far.  BUT: The CDDB disc-id is based on the real
-						 * TOC entries so this adjustment would result in a wrong Disc-ID.
-						 * We can only solve this conflict, when we save the old
-						 * (toc-based) start sector of the data track.  Of course the
-						 * correct solution would be, to only adjust the _length_ of the
-						 * last audio track, not the start of the next track, but the
-						 * internal structures of cdparanoia are as they are, so the
-						 * length is only implicitely given.  Bloody sh*. */
-						start_of_first_data_as_in_toc = d->disc_toc[j].dwStartSector;
-						hack_track = j + 1;
-						d->disc_toc[j].dwStartSector = ms_addr - 11400;
-					}
-					break;
-				}
-			return 1;
-		}
-	}
-	return 0;
-}
-#endif
-
-/* libcdda returns for cdda_disc_lastsector() the last sector of the last
-	 _audio_ track.  How broken. For CDDB Disc-ID we need the real last sector
-	 to calculate the disc length.  */
-long my_last_sector(cdrom_drive *drive)
-{
-	return cdda_track_lastsector(drive, drive->tracks);
-}
-
-/* Stupid CDparanoia returns the first sector of the first _audio_ track
-	 as first disc sector.  Equally broken to the last sector.  But what is
-	 even more shitty is, that if it happens that the first audio track is
-	 the first track at all, it returns a hardcoded _zero_, whatever else
-	 the TOC told it. This of course happens quite often, as usually the first
-	 track is audio, if there's audio at all. And usually it even works,
-	 because most of the time the real TOC offset is 150 frames, which we
-	 accounted for in our code. This is so unbelievable ugly. */
-long my_first_sector(cdrom_drive *drive)
-{
-	return cdda_track_firstsector(drive, 1);
-}
-
 using namespace AudioCD;
 
 static const KCmdLineOptions options[] =
@@ -205,11 +73,9 @@ static const KCmdLineOptions options[] =
     KCmdLineLastOption
 };
 
-int
-kdemain(int argc, char ** argv)
+int kdemain(int argc, char ** argv)
 {
-	// KApplication is used as libkcddb uses ioslaves which need a valid
-	// kapp pointer
+	// KApplication uses libkcddb which needs a valid kapp pointer
 	// GUIenabled must be true as libkcddb sometimes wants to communicate
 	// with the user
 	putenv(strdup("SESSION_MANAGER="));
@@ -238,9 +104,8 @@ enum Which_dir {
 
 class AudioCDProtocol::Private {
 public:
-	Private() {
+	Private() : cd(KCompactDisc::Asynchronous) {
 		clearURLargs();
-		discid = "";
 		s_info = i18n("Information");
 		s_fullCD = i18n("Full CD");
 	}
@@ -263,15 +128,16 @@ public:
 	QString device; // URL settable
 	int paranoiaLevel; // URL settable
 	bool reportErrors;
-	
+
 	// Directory strings, never change after init
 	QString s_info;
 	QString s_fullCD;
 
 	// Current CD
-	QString discid;
-	uint tracks;
+	unsigned discid;
+	unsigned tracks;
 	bool trackIsAudio[100];
+	KCompactDisc cd; // keep it around so that we don't assume the disk changed between every stat()
 
 	// CDDB items
 	KCDDB::CDDB::Result cddbResult;
@@ -286,27 +152,19 @@ public:
 	QString rreplace;
 
 	// Current strings for this CD and or cddb selection
-	QString cd_title;
-	QString cd_artist;
-	QString cd_category;
-	int cd_year;
 	QStringList templateTitles;
 	QString templateAlbumName;
-
-	// The current cddb title strings for id3
-	QStringList track_titles;
 };
 
 int paranoia_read_limited_error;
 
-
 AudioCDProtocol::AudioCDProtocol(const QCString & protocol, const QCString & pool, const QCString & app)
 	: SlaveBase(protocol, pool, app)
 {
 	d = new Private;
 
 	// Add encoders
-	AudioCDEncoder::find_all_plugins(this, encoders);
+	AudioCDEncoder::findAllPlugins(this, encoders);
 	encoderTypeCDA = encoderFromExtension(".cda");
 	encoderTypeWAV = encoderFromExtension(".wav");
 	encoders.setAutoDelete(true);
@@ -345,47 +203,82 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 		return 0;
 	}
 
-	// Tell encoders to read their settings.
-	AudioCDEncoder *encoder;
-	for ( encoder = encoders.first(); encoder; encoder = encoders.next() )
-		encoder->init();
-
 	// Load OUR Settings.
 	loadSettings();
-
 	// Then url parameters can overrule our settings.
 	parseURLArgs(url);
 
-	struct cdrom_drive * drive = pickDrive();
-
+	struct cdrom_drive * drive = getDrive();
 	if (0 == drive)
 		return 0;
 
-	if (0 != cdda_open(drive))
+	// Update our knowledge of the disc
+	// TODO which one is right?
+	// qDebug("\"%s\" \"%s\"", drive->cdda_device_name, drive->ioctl_device_name);
+	d->cd.setDevice(drive->cdda_device_name, 50, false);
+#if 0
+	// FreeBSD's cdparanoia as of january 5th 2006 has rather broken
+	// support for non-SCSI devices. Although it finds ATA cdroms just
+	// fine, there is no straightforward way to discover the device
+	// name associated with the device, which throws the rest of audiocd
+	// for a loop.
+	//
+	if ( !(drive->dev) || (COOKED_IOCTL == drive->interface) )
 	{
-		kdDebug(7117) << "cdda_open failed" << endl;
+		// For ATAPI devices, we have no real choice. Use the
+		// user selected value, even if there is none.
+		//
+		kdWarning(7117) << "Found an ATAPI device, assuming it is the one specified by the user." << endl;
+		d->cd.setDevice( d->device );
+	}
+	else
+	{
+		kdDebug(7117) << "Found a SCSI or ATAPICAM device." << endl;
+		if ( strlen(drive->dev->device_path) > 0 )
+		{
+			d->cd.setDevice( drive->dev->device_path );
+		}
+		else
+		{
+			// But the device_path can be empty under some
+			// circumstances, so build a representation from
+			// the unit number and SCSI device name.
+			//
+			QString devname = QString::fromLatin1( "/dev/%1%2" )
+				.arg( drive->dev->given_dev_name )
+				.arg( drive->dev->given_unit_number ) ;
+			kdDebug(7117) << "  Using derived name " << devname << endl;
+			d->cd.setDevice( devname );
+		}
+	}
+#endif
 
-		error(KIO::ERR_CANNOT_OPEN_FOR_READING, url.path());
-		return 0;
+	if (d->cd.discId() != d->discid && d->cd.discId() != d->cd.missingDisc){
+		d->discid = d->cd.discId();
+		d->tracks = d->cd.tracks();
+		for(uint i=0; i< d->cd.tracks(); i++)
+			d->trackIsAudio[i] = d->cd.isAudio(i+1);
+
+		KCDDB::Client c;
+		d->cddbResult = c.lookup(d->cd.discSignature());
+		d->cddbList = c.lookupResponse();
+		d->cddbBestChoice = c.bestLookupResponse();
+		generateTemplateTitles();
 	}
 
-	updateCD(drive);
-
-	//
 	// Determine what file or folder that is wanted.
-	//
-
 	d->fname = url.fileName(false);
 	QString dname = url.directory(true, false);
 	if (!dname.isEmpty() && dname[0] == '/')
 		dname = dname.mid(1);
 
-	// Kong bug where they send dirs as files, double check
+	// Kong issue where they send dirs as files, double check
 	/* A hack, for when konqi wants to list the directory audiocd:/Bla
 		 it really submits this URL, instead of audiocd:/Bla/ to us. We could
 		 send (in listDir) the UDS_NAME as "Bla/" for directories, but then
 		 konqi shows them as "Bla//" in the status line. */
 	// See if it is an encoder directory
+	AudioCDEncoder *encoder;
 	for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
 		if(encoder->type() == d->fname){
 			dname = d->fname;
@@ -394,13 +287,11 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 		}
 	}
 	// Other Hard coded directories
-	if (dname.isEmpty() &&
-			(d->fname == d->s_info || d->fname == QFL1("Information") ||
-			 d->fname == d->s_fullCD ))
-		{
-			dname = d->fname;
-			d->fname = "";
-		}
+	if (dname.isEmpty() && (d->fname == d->s_info || d->fname == d->s_fullCD ))
+	{
+		dname = d->fname;
+		d->fname = "";
+	}
 	// end hack
 
 
@@ -416,9 +307,9 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 	if ( Unknown == d->which_dir ){
 		if (dname.isEmpty())
 			d->which_dir = Root;
-		else if (dname == d->s_info || dname == QFL1("Information"))
+		else if (dname == d->s_info)
 			d->which_dir = Info;
-		else if (dname == d->s_fullCD || dname == QFL1("Full CD"))
+		else if (dname == d->s_fullCD)
 			d->which_dir = FullCD;
 	}
 
@@ -467,7 +358,7 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 		d->req_track = -1;
 
 	// Are we in the directory that lists "full CD" files?
-	d->req_allTracks = (dname.contains(d->s_fullCD) || dname.contains(QFL1("Full CD")));
+	d->req_allTracks = (dname.contains(d->s_fullCD));
 
 	kdDebug(7117) << "dir=" << dname << " file=" << d->fname
 		<< " req_track=" << d->req_track << " which_dir=" << d->which_dir << " full CD?=" << d->req_allTracks << endl;
@@ -543,25 +434,25 @@ void AudioCDProtocol::get(const KURL & url)
 
 	AudioCDEncoder *encoder = determineEncoder(d->fname);
 	if(!encoder){
-	 cdda_close(drive);
-	 return;
+		cdda_close(drive);
+		return;
 	}
 
+	KCDDB::CDInfo info;
 	if(d->cddbResult == KCDDB::CDDB::Success){
-		QString trackName;
-		// do we rip the whole CD?
-		if (d->req_allTracks)
-			// YES => the title of the file is the title of the CD
-			trackName = d->cd_title.utf8().data();
-		else
-			// Passing the actual track title and not the template to the encoder for id3 type info
-			// NO => title of the track.
-			trackName = d->track_titles[d->req_track];
+		info = d->cddbBestChoice;
 
-		encoder->fillSongInfo(trackName, d->cd_artist, d->cd_title, d->cd_category, d->req_track+1, d->cd_year, "");
-	}
-	else {
-		encoder->fillSongInfo("", "", "", "", d->req_track+1, 0, QString("CDDBDISCID=%1").arg(d->discid));
+		int track = d->req_track;
+
+		QString trackName;
+		// hack
+		// do we rip the whole CD?
+		if (d->req_allTracks){
+			track = 0;
+			// YES => the title of the file is the title of the CD
+			info.trackInfoList[track].title = info.title.utf8().data();
+		}
+		encoder->fillSongInfo(info, track, "");
 	}
 	long totalByteCount = CD_FRAMESIZE_RAW * (lastSector - firstSector + 1);
 	long time_secs = (8 * totalByteCount) / (44100 * 2 * 16);
@@ -644,61 +535,6 @@ void AudioCDProtocol::stat(const KURL & url)
 	finished();
 }
 
-void AudioCDProtocol::updateCD(struct cdrom_drive * drive)
-{
-	d->tracks = cdda_tracks(drive);
-
-	KCDDB::TrackOffsetList qvl;
-
-	for(uint i=0; i< d->tracks; i++){
-			d->trackIsAudio[i] = cdda_track_audiop(drive, i + 1);
-			// Does this ever happen??
-			// kdDebug(7117) << "Hack track: " << hack_track << endl;
-			if (((int)i+1) != hack_track)
-				qvl.append(cdda_track_firstsector(drive, i + 1) + 150);
-			else
-				qvl.append(start_of_first_data_as_in_toc + 150);
-	}
-
-	qvl.append(my_first_sector(drive)+150);
-	qvl.append(my_last_sector(drive)+150);
-
-	KCDDB::CDDB cddb;
-	QString id = cddb.trackOffsetListToId(qvl);
-	if (id == d->discid){
-		return;
-	}
-	d->discid = id;
-	d->templateTitles.clear();
-	d->track_titles.clear();
-	KCDDB::Client c;
-
-	d->cddbResult = c.lookup(qvl);
-
-	if (d->cddbResult == KCDDB::CDDB::Success)
-	{
-		d->cddbList = c.lookupResponse();
-		d->cddbBestChoice = c.bestLookupResponse();
-		generateTemplateTitles();
-		return;
-	}
-
-	for (uint i = 1; i <= d->tracks; i++)
-	{
-		QString s;
-		QString num;
-		num.sprintf("%02d", i);
-		if (cdda_track_audiop(drive, i))
-			s = i18n("Track %1").arg(num);
-		else
-			s.sprintf("data%02d", i);
-		d->templateTitles.append( s );
-
-		// keep both lists in sync
-		d->track_titles.append( QString::null );
-	}
-}
-
 static void app_entry(UDSEntry& e, unsigned int uds, const QString& str)
 {
 	UDSAtom a;
@@ -805,14 +641,13 @@ void AudioCDProtocol::listDir(const KURL & url)
 	}
 
 	// Now fill in the tracks for the current directory
-
 	if (list_tracks && d->which_dir == FullCD) {
 		// if we're listing the "full CD" subdirectory :
 		if ( (d->which_dir == FullCD) ) {
 			AudioCDEncoder *encoder;
 			for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
 				if (d->cddbResult != KCDDB::CDDB::Success)
-					addEntry(QFL1("Full CD"), encoder, drive, -1);
+					addEntry(d->s_fullCD, encoder, drive, -1);
 				else
 					addEntry(d->templateAlbumName, encoder, drive, -1);
 			}
@@ -887,7 +722,7 @@ long AudioCDProtocol::fileSize(long firstSector, long lastSector, AudioCDEncoder
 	return encoder->size(length_seconds);
 }
 
-struct cdrom_drive *AudioCDProtocol::pickDrive()
+struct cdrom_drive *AudioCDProtocol::getDrive()
 {
 	QCString device(QFile::encodeName(d->device));
 
@@ -907,23 +742,26 @@ struct cdrom_drive *AudioCDProtocol::pickDrive()
 	}
 
 	if (0 == drive) {
-		kdDebug(7117) << "Can't find an audio CD" << endl;
+		kdDebug(7117) << "Can't find an audio CD on: \"" << d->device << "\"" << endl;
 
-		if(!QFile::exists(d->device))
+		QFileInfo fi(d->device);
+		if(!fi.isReadable())
+			error(KIO::ERR_SLAVE_DEFINED, i18n("Device doesn't have read permissions for this account.  Check the read permissions on the device."));
+		else if(!fi.isWritable())
+			error(KIO::ERR_SLAVE_DEFINED, i18n("Device doesn't have write permissions for this account.  Check the write permissions on the device."));
+		else if(!fi.exists())
 			error(KIO::ERR_DOES_NOT_EXIST, d->device);
-		else {
-			QFileInfo fi(d->device);
-			if(!fi.isReadable())
-				error(KIO::ERR_CANNOT_OPEN_FOR_READING, d->device);
-			else if(!fi.isWritable())
-				error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, d->device);
-#if KDE_IS_VERSION (3, 3, 89)
-			else error(KIO::ERR_SLAVE_DEFINED, 
+		else error(KIO::ERR_SLAVE_DEFINED,
 i18n("Unknown error.  If you have a cd in the drive try running cdparanoia -vsQ as yourself (not root). Do you see a track list? If not, make sure you have permission to access the CD device. If you are using SCSI emulation (possible if you have an IDE CD writer) then make sure you check that you have read and write permissions on the generic SCSI device, which is probably /dev/sg0, /dev/sg1, etc.. If it still does not work, try typing audiocd:/?device=/dev/sg0 (or similar) to tell kio_audiocd which device your CD-ROM is."));
-#else			
-			else error(KIO::ERR_UNKNOWN, d->device);
-#endif
-		}
+		return 0;
+	}
+
+	if (0 != cdda_open(drive))
+	{
+		kdDebug(7117) << "cdda_open failed" << endl;
+		error(KIO::ERR_CANNOT_OPEN_FOR_READING, d->device);
+		cdda_close(drive);
+		return 0;
 	}
 
 	return drive;
@@ -1004,7 +842,11 @@ void AudioCDProtocol::paranoiaRead(
 		if(encoderProcessed == -1){
 			kdDebug(7117) << "Encoder processing error, stopping." << endl;
 			ok = false;
-			error( ERR_SLAVE_DEFINED, i18n( "Couldn't read %1: encoding failed" ).arg( fileName ) );
+			QString errMsg = i18n( "Couldn't read %1: encoding failed" ).arg( fileName );
+			QString details = encoder->lastErrorMessage();
+			if ( !details.isEmpty() )
+			    errMsg += "\n" + details;
+			error( ERR_SLAVE_DEFINED, errMsg );
 			break;
 		}
 		processed += encoderProcessed;
@@ -1054,7 +896,7 @@ void AudioCDProtocol::paranoiaRead(
 					margin = 7;
 				unsigned long low = lastSize - lastSize/margin;
 				if(estSize < low){
-					//qDebug("low: %ld, estSize: %ld, num: %i", low, estSize, num);
+					//qDebug("low: %ld, estSize: %ld, num: %i", low, estSize, margin);
 					totalSize( estSize );
 					lastSize = estSize;
 				}
@@ -1063,7 +905,7 @@ void AudioCDProtocol::paranoiaRead(
 		/**
 		 * End estimation.
 		 */
-
+		//qDebug("processed: %d, totalSize: %d", processed, estSize);
 		processedSize(processed);
 	}
 
@@ -1173,6 +1015,7 @@ void AudioCDProtocol::loadSettings()
 	// Tell the encoders to load their settings
 	AudioCDEncoder *encoder;
 	for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
+		encoder->init(); // ### TODO check return value!
 		encoder->loadSettings();
 	}
 
@@ -1184,34 +1027,32 @@ void AudioCDProtocol::loadSettings()
  */
 void AudioCDProtocol::generateTemplateTitles()
 {
+	d->templateTitles.clear();
 	if (d->cddbResult != KCDDB::CDDB::Success)
+	{
+		for (unsigned int i = 0; i < d->tracks; i++){
+			QString n;
+			d->templateTitles.append( i18n("Track %1").arg(n.sprintf("%02d", i + 1)));
+		}
 		return;
+	}
 
 	KCDDB::CDInfo info = d->cddbBestChoice;
 	if(d->cddbUserChoice >= 0 && (((uint)d->cddbUserChoice) < d->cddbList.count()))
 		info = d->cddbList[d->cddbUserChoice];
 
-	// First save it for the encoders
-	// Might it be easier to just save the whole thing?
-	d->cd_title = info.title;
-	d->cd_artist = info.artist;
-	d->cd_category = info.genre;
-	d->cd_year = info.year;
-	KCDDB::TrackInfoList t = info.trackInfoList;
-	for (uint i = 0; i < t.count(); i++)
-		d->track_titles.append(t[i].title);
-
 	// Then generate the templates
 	d->templateTitles.clear();
 	for (uint i = 0; i < d->tracks; i++) {
 		QMap<QString, QString> macros;
-		macros["albumartist"] = d->cd_artist;
-		macros["albumtitle"] = d->cd_title;
-		macros["title"] = d->track_titles[i];
+		macros["albumartist"] = info.artist;
+		macros["albumtitle"] = info.title;
+		macros["title"] = (info.trackInfoList[i].title);
 		QString n;
 		macros["number"] = n.sprintf("%02d", i + 1);
-		macros["genre"] = d->cd_category;
-		macros["year"] = QString::number(d->cd_year);
+		//macros["number"] = QString("%1").arg(i+1, 2, 10);
+		macros["genre"] = info.genre;
+		macros["year"] = QString::number(info.year);
 
 		QString title = KMacroExpander::expandMacros(d->fileNameTemplate, macros, '%').replace('/', QFL1("%2F"));
 		title.replace( QRegExp(d->rsearch), d->rreplace );
@@ -1219,10 +1060,10 @@ void AudioCDProtocol::generateTemplateTitles()
 	}
 
 	QMap<QString, QString> macros;
-	macros["albumartist"] = d->cd_artist;
-	macros["albumtitle"] = d->cd_title;
-	macros["genre"] = d->cd_category;
-	macros["year"] = QString::number(d->cd_year);
+	macros["albumartist"] = info.artist;
+	macros["albumtitle"] = info.title;
+	macros["genre"] = info.genre;
+	macros["year"] = QString::number(info.year);
 	d->templateAlbumName = KMacroExpander::expandMacros(d->albumTemplate, macros, '%').replace('/', QFL1("%2F"));
 	d->templateAlbumName.replace( QRegExp(d->rsearch), d->rreplace );
 }
