@@ -13,18 +13,7 @@
 #include <qtimer.h>
 #include <kdelibs_export.h>
 
-extern "C"
-{
-  /**
-   * This function is the 'main' function of this part.  It takes
-   * the form 'void *init_lib<library name>()'.  It always returns a
-   * new factory object
-   */
-  KDE_EXPORT void *init_libkmidpart()
-  {
-    return new KMidFactory;
-  }
-}
+K_EXPORT_COMPONENT_FACTORY( libkmidpart, KMidFactory )
 
 /**
  * We need one static instance of the factory for our C 'main'
