@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     Send comments and bug fixes to larrosa@kde.org
     or to Antonio Larrosa, Rio Arnoya, 10 5B, 29006 Malaga, Spain
@@ -122,7 +122,7 @@ void MidiConfigDialog::browseMap()
 
   delete selectedmap;
   selectedmap=new char[filename.length()+1];
-  strcpy(selectedmap,filename.ascii());
+  strcpy(selectedmap,QFile::encodeName(filename));
   maplabel->setText(selectedmap);
 }
 
