@@ -1,8 +1,6 @@
 /*
  * KMix -- KDE's full featured mini mixer
  *
- * $Id: kmix.cpp 408616 2005-04-29 13:41:05Z lunakl $
- *
  * Copyright (C) 2000 Stefan Schimanski <schimmi@kde.org>
  * Copyright (C) 2001 Preston Brown <pbrown@kde.org>
  * Copyright (C) 2003 Sven Leiber <s.leiber@web.de>
@@ -46,6 +44,8 @@
 #include <khelpmenu.h>
 #include <kdebug.h>
 #include <kaccel.h>
+#include <kglobalaccel.h>
+#include <kkeydialog.h>
 #include <kpopupmenu.h>
 
 // application specific includes
@@ -62,18 +62,18 @@
  */
 KMixWindow::KMixWindow()
 	: KMainWindow(0, 0, 0, 0), m_showTicks( true ),
-	m_lockedLayout(0),
 	m_dockWidget( 0L )
 {
 	m_visibilityUpdateAllowed	= true;
 	m_multiDriverMode		= false; // -<- I never-ever want the multi-drivermode to be activated by accident
-	m_surroundView		        = false; // -<- The same is true for the experimental surround View (3D)
+	m_surroundView		        = false; // -<- Also the experimental surround View (3D)
+	m_gridView		        = false; // -<- Also the experimental Grid View
 	// As long as we do not know better, we assume to start hidden. We need
 	// to initialize this variable here, as we don't trigger a hideEvent().
 	m_isVisible = false;
 	m_mixerWidgets.setAutoDelete(true);
 	loadConfig(); // Need to load config before initMixer(), due to "MultiDriver" keyword
-	MixerToolBox::initMixer(m_mixers, m_multiDriverMode, m_hwInfoString);
+	MixerToolBox::initMixer(Mixer::mixers(), m_multiDriverMode, m_hwInfoString);
 	initActions();
 	initWidgets();
 	initMixerWidgets();
@@ -99,6 +99,7 @@ KMixWindow::KMixWindow()
 
 KMixWindow::~KMixWindow()
 {
+   MixerToolBox::deinitMixer();
 }
 
 
@@ -111,14 +112,23 @@ KMixWindow::initActions()
 	// settings menu
 	KStdAction::showMenubar( this, SLOT(toggleMenuBar()), actionCollection());
 	KStdAction::preferences( this, SLOT(showSettings()), actionCollection());
+	new KAction( i18n( "Configure &Global Shortcuts..." ), "configure_shortcuts", 0, this,
+		SLOT( configureGlobalShortcuts() ), actionCollection(), "settings_global" );
 	KStdAction::keyBindings( guiFactory(), SLOT(configureShortcuts()), actionCollection());
 
-/* Belongs in KMixDockWidget
-	(void)new KToggleAction( i18n( "M&ute" ), 0, this, SLOT( dockMute() ),
-				 actionCollection(), "dock_mute" );
-*/
 	(void) new KAction( i18n( "Hardware &Information" ), 0, this, SLOT( slotHWInfo() ), actionCollection(), "hwinfo" );
 	(void) new KAction( i18n( "Hide Mixer Window" ), Key_Escape, this, SLOT(hide()), actionCollection(), "hide_kmixwindow" );
+
+	m_globalAccel = new KGlobalAccel( this );
+	m_globalAccel->insert( "Increase volume", i18n( "Increase Volume of Master Channel"), QString::null,
+			KShortcut(), KShortcut(), this, SLOT( increaseVolume() ) );
+	m_globalAccel->insert( "Decrease volume", i18n( "Decrease Volume of Master Channel"), QString::null,
+			KShortcut(), KShortcut(), this, SLOT( decreaseVolume() ) );
+	m_globalAccel->insert( "Toggle mute", i18n( "Toggle Mute of Master Channel"), QString::null,
+			KShortcut(), KShortcut(), this, SLOT( toggleMuted() ) );
+	m_globalAccel->readSettings();
+	m_globalAccel->updateConnections();
+
 	createGUI( "kmixui.rc" );
 }
 
@@ -149,7 +159,7 @@ KMixWindow::initWidgets()
 	QSizePolicy qsp( QSizePolicy::Ignored, QSizePolicy::Maximum);
 	mixerNameLayout->setSizePolicy(qsp);
 	mixerNameLayout->setSpacing(KDialog::spacingHint());
-	QLabel *qlbl = new QLabel( i18n(" Current mixer:"), mixerNameLayout );
+	QLabel *qlbl = new QLabel( i18n("Current mixer:"), mixerNameLayout );
 	qlbl->setFixedHeight(qlbl->sizeHint().height());
 	m_cMixer = new KComboBox( FALSE, mixerNameLayout, "mixerCombo" );
 	m_cMixer->setFixedHeight(m_cMixer->sizeHint().height());
@@ -176,18 +186,17 @@ void
 KMixWindow::updateDocking()
 {
 	// delete old dock widget
-	if (m_dockWidget)
+	if (!m_showDockWidget)
 	{
 		delete m_dockWidget;
 		m_dockWidget = 0L;
 	}
-
-	if (m_showDockWidget)
+	else if (m_showDockWidget && !m_dockWidget)
 	{
 
 		// create dock widget
                 // !! This should be a View in the future
-		m_dockWidget = new KMixDockWidget( m_mixers.first(), this, "mainDockWidget", m_volumeWidget );
+		m_dockWidget = new KMixDockWidget( Mixer::mixers().first(), this, "mainDockWidget", m_volumeWidget );
 
 /* Belongs in KMixDockWidget
 		// create RMB menu
@@ -205,6 +214,7 @@ KMixWindow::updateDocking()
 		 */
 		QString selectChannel = i18n("Select Channel"); // This text will be used in KDE3.4.1 !!!
 
+		m_dockWidget->updatePixmap();
 		m_dockWidget->show();
 	}
 }
@@ -239,6 +249,22 @@ KMixWindow::saveConfig()
    config->writeEntry( "Tickmarks", m_showTicks );
    config->writeEntry( "Labels", m_showLabels );
    config->writeEntry( "startkdeRestore", m_onLogin );
+   Mixer* mixerMasterCard = Mixer::masterCard();
+   if ( mixerMasterCard != 0 ) {
+      config->writeEntry( "MasterMixer", mixerMasterCard->id() );
+   }
+   MixDevice* mdMaster = Mixer::masterCardDevice();
+   if ( mdMaster != 0 ) {
+      config->writeEntry( "MasterMixerDevice", mdMaster->getPK() );
+   }
+
+   if ( m_valueStyle == MixDeviceWidget::NABSOLUTE )
+      config->writeEntry( "ValueStyle", "Absolute");
+   else if ( m_valueStyle == MixDeviceWidget::NRELATIVE )
+      config->writeEntry( "ValueStyle", "Relative");
+   else 
+     config->writeEntry( "ValueStyle", "None" );
+
    if ( m_toplevelOrientation  == Qt::Vertical )
       config->writeEntry( "Orientation","Vertical" );
    else
@@ -267,11 +293,25 @@ KMixWindow::loadConfig()
    m_hideOnClose = config->readBoolEntry("HideOnClose", true);
    m_showTicks = config->readBoolEntry("Tickmarks", true);
    m_showLabels = config->readBoolEntry("Labels", true);
+   const QString& valueStyleString = config->readEntry("ValueStyle", "None");
    m_onLogin = config->readBoolEntry("startkdeRestore", true );
    m_startVisible = config->readBoolEntry("Visible", true);
    m_multiDriverMode = config->readBoolEntry("MultiDriver", false);
    m_surroundView    = config->readBoolEntry("Experimental-ViewSurround", false );
+   m_gridView    = config->readBoolEntry("Experimental-ViewGrid", false );
    const QString& orientationString = config->readEntry("Orientation", "Horizontal");
+   QString mixerMasterCard = config->readEntry( "MasterMixer", "" );
+   Mixer::setMasterCard(mixerMasterCard);
+   QString masterDev = config->readEntry( "MasterMixerDevice", "" );
+   Mixer::setMasterCardDevice(masterDev);
+
+   if ( valueStyleString == "Absolute" )
+       m_valueStyle = MixDeviceWidget::NABSOLUTE;
+   else if ( valueStyleString == "Relative" )
+       m_valueStyle = MixDeviceWidget::NRELATIVE;
+   else 
+       m_valueStyle = MixDeviceWidget::NNONE;
+
    if ( orientationString == "Vertical" )
        m_toplevelOrientation  = Qt::Vertical;
    else
@@ -305,40 +345,35 @@ KMixWindow::initMixerWidgets()
 	int id=0;
 	Mixer *mixer;
 
-	// Attention!! If m_mixers is empty, we behave stupid. We don't show nothing and there
+	// Attention!! If Mixer::mixers() is empty, we behave stupid. We don't show nothing and there
         //             is not even a context menu.
-	for ( mixer=m_mixers.first(),id=0; mixer!=0; mixer=m_mixers.next(),id++ )
+	for ( mixer=Mixer::mixers().first(),id=0; mixer!=0; mixer=Mixer::mixers().next(),id++ )
 	{
-	    //kdDebug(67100) << "Mixer number: " << id << " Name: " << mixer->mixerName() << endl ;
-
-
-                ViewBase::ViewFlags vflags = ViewBase::HasMenuBar;
-                if ( m_showMenubar ) {
-                    vflags |= ViewBase::MenuBarVisible;
-	        }
+		//kdDebug(67100) << "Mixer number: " << id << " Name: " << mixer->mixerName() << endl ;
+		ViewBase::ViewFlags vflags = ViewBase::HasMenuBar;
+		if ( m_showMenubar ) {
+			vflags |= ViewBase::MenuBarVisible;
+		}
 		if (  m_surroundView ) {
-		    vflags |= ViewBase::Experimental_SurroundView;
+			vflags |= ViewBase::Experimental_SurroundView;
+		}
+		if (  m_gridView ) {
+			vflags |= ViewBase::Experimental_GridView;
 		}
 		if ( m_toplevelOrientation == Qt::Vertical ) {
-		    vflags |= ViewBase::Vertical;
+			vflags |= ViewBase::Vertical;
 		}
 		else {
-		    vflags |= ViewBase::Horizontal;
+			vflags |= ViewBase::Horizontal;
 		}
 
 	
-		KMixerWidget *mw = new KMixerWidget( id, mixer, mixer->mixerName(), mixer->mixerNum(),
+		KMixerWidget *mw = new KMixerWidget( id, mixer, mixer->mixerName(),
 						     MixDevice::ALL, this, "KMixerWidget", vflags );
-
-		//mw->setName( mixer->mixerName() );
-
 		m_mixerWidgets.append( mw );
 
-		// Add to Combo
+		// Add to Combo and Stack
 		m_cMixer->insertItem( mixer->mixerName() );
-
-		// Add to Stack
-		//kdDebug(67100) << "Inserted mixer " << id << ":" << mw->name() << endl;
 		m_wsMixers->addWidget( mw, id );
 
 		QString grp;
@@ -347,6 +382,7 @@ KMixWindow::initMixerWidgets()
 
 		mw->setTicks( m_showTicks );
 		mw->setLabels( m_showLabels );
+		mw->setValueStyle ( m_valueStyle );
 		// !! I am still not sure whether this works 100% reliably - chris
 		mw->show();
 	}
@@ -358,6 +394,8 @@ KMixWindow::initMixerWidgets()
 	}
 }
 
+
+
 bool
 KMixWindow::queryClose ( )
 {
@@ -389,6 +427,9 @@ KMixWindow::showSettings()
       m_prefDlg->m_onLogin->setChecked( m_onLogin );
       m_prefDlg->_rbVertical  ->setChecked( m_toplevelOrientation == Qt::Vertical );
       m_prefDlg->_rbHorizontal->setChecked( m_toplevelOrientation == Qt::Horizontal );
+      m_prefDlg->_rbNone->setChecked( m_valueStyle == MixDeviceWidget::NNONE );
+      m_prefDlg->_rbAbsolute->setChecked( m_valueStyle == MixDeviceWidget::NABSOLUTE );
+      m_prefDlg->_rbRelative->setChecked( m_valueStyle == MixDeviceWidget::NRELATIVE );
 
       m_prefDlg->show();
    }
@@ -419,7 +460,7 @@ void
 KMixWindow::loadVolumes()
 {
 	KConfig *cfg = new KConfig( "kmixctrlrc", true );
-	for (Mixer *mixer=m_mixers.first(); mixer!=0; mixer=m_mixers.next())
+	for (Mixer *mixer=Mixer::mixers().first(); mixer!=0; mixer=Mixer::mixers().next())
 	{
 		mixer->volumeLoad( cfg );
 	}
@@ -435,7 +476,7 @@ void
 KMixWindow::saveVolumes()
 {
     KConfig *cfg = new KConfig( "kmixctrlrc", false );
-    for (Mixer *mixer=m_mixers.first(); mixer!=0; mixer=m_mixers.next()) {
+    for (Mixer *mixer=Mixer::mixers().first(); mixer!=0; mixer=Mixer::mixers().next()) {
 	//kdDebug(67100) << "KMixWindow::saveConfig()" << endl;
 	mixer->volumeSave( cfg );
     }
@@ -451,12 +492,29 @@ KMixWindow::applyPrefs( KMixPrefDlg *prefDlg )
    m_showTicks = prefDlg->m_showTicks->isChecked();
    m_showLabels = prefDlg->m_showLabels->isChecked();
    m_onLogin = prefDlg->m_onLogin->isChecked();
+
+   if ( prefDlg->_rbNone->isChecked() ) {
+     m_valueStyle = MixDeviceWidget::NNONE;
+   } else if ( prefDlg->_rbAbsolute->isChecked() ) {
+     m_valueStyle = MixDeviceWidget::NABSOLUTE;
+   } else if ( prefDlg->_rbRelative->isChecked() ) {
+     m_valueStyle = MixDeviceWidget::NRELATIVE;
+   }
+
+   bool toplevelOrientationHasChanged =
+        ( prefDlg->_rbVertical->isChecked()   && m_toplevelOrientation == Qt::Horizontal )
+     || ( prefDlg->_rbHorizontal->isChecked() && m_toplevelOrientation == Qt::Vertical   );
+   if ( toplevelOrientationHasChanged ) {
+      QString msg = i18n("The change of orientation will be adopted on the next start of KMix.");
+      KMessageBox::information(0,msg);
+   }
    if ( prefDlg->_rbVertical->isChecked() ) {
-      kdDebug(67100) << "KMix should change to Vertical layout\n";
+      //QString "For a change of language to take place, quit and restart KDiff3.";
+      //kdDebug(67100) << "KMix should change to Vertical layout\n";
       m_toplevelOrientation = Qt::Vertical;
    }
    else if ( prefDlg->_rbHorizontal->isChecked() ) {
-     kdDebug(67100) << "KMix should change to Horizontal layout\n";
+     //kdDebug(67100) << "KMix should change to Horizontal layout\n";
      m_toplevelOrientation = Qt::Horizontal;
    }
 
@@ -468,10 +526,11 @@ KMixWindow::applyPrefs( KMixPrefDlg *prefDlg )
    {
       mw->setTicks( m_showTicks );
       mw->setLabels( m_showLabels );
+      mw->setValueStyle ( m_valueStyle );
       mw->mixer()->readSetFromHWforceUpdate(); // needed, as updateDocking() has reconstructed the DockWidget
    }
 
-   this->setUpdatesEnabled(false);
+   this->setUpdatesEnabled(true);
 
    // avoid invisible and unaccessible main window
    if( !m_showDockWidget && !isVisible() )
@@ -536,5 +595,49 @@ KMixWindow::showSelectedMixer( int mixer )
 	m_wsMixers->raiseWidget( mixer );
 }
 
+void
+KMixWindow::configureGlobalShortcuts()
+{
+	KKeyDialog::configure( m_globalAccel, 0, false ) ;
+        m_globalAccel->writeSettings();
+        m_globalAccel->updateConnections();
+}
+
+void
+KMixWindow::toggleMuted()
+{
+   Mixer* mixerMaster = Mixer::masterCard();
+   if ( mixerMaster != 0 ) {
+      MixDevice* md = mixerMaster->masterDevice();
+      if ( md != 0 && md->hasMute() ) {
+         mixerMaster->toggleMute(md->num());
+      }
+   }
+}
+
+void
+KMixWindow::increaseVolume()
+{
+   Mixer* mixerMaster = Mixer::masterCard();
+   if ( mixerMaster != 0 ) {
+      MixDevice* md = mixerMaster->masterDevice();
+      if ( md != 0 ) {
+         mixerMaster->increaseVolume(md->num());
+      }
+   }
+}
+
+void
+KMixWindow::decreaseVolume()
+{
+   Mixer* mixerMaster = Mixer::masterCard();
+   if ( mixerMaster != 0 ) {
+      MixDevice* md = mixerMaster->masterDevice();
+      if ( md != 0 ) {
+         mixerMaster->decreaseVolume(md->num());
+      }
+   }
+}
+
 #include "kmix.moc"
 
