@@ -25,8 +25,8 @@
 #include "mixer.h"
 #include "mixdevicewidget.h"
 
-ViewInput::ViewInput(QWidget* parent, const char* name, Mixer* mixer, ViewBase::ViewFlags vflags)
-      : ViewSliders(parent, name, mixer, vflags)
+ViewInput::ViewInput(QWidget* parent, const char* name, const QString & caption, Mixer* mixer, ViewBase::ViewFlags vflags)
+      : ViewSliders(parent, name, caption, mixer, vflags)
 {
     init();
     connect ( _mixer, SIGNAL(newRecsrc())      , this, SLOT(refreshVolumeLevels()) ); // only the input widget has record sources
