@@ -90,6 +90,14 @@ namespace KCDDB
     return *this;
   }
 
+  QVariant TrackInfo::get(const QString &type) const {
+    if(type == "title")
+      return title;
+    if(type == "extt")
+      return extt;
+    return QVariant();
+  }
+  
     bool
   CDInfo::load(const QString & s)
   {
@@ -257,7 +265,7 @@ namespace KCDDB
     QString lines;
 
     while (tmpValue.length() > maxLength)
-      {
+    {
       lines += QString("%1=%2\n").arg(name,tmpValue.left(maxLength));
       tmpValue = tmpValue.mid(maxLength);
     }
@@ -319,6 +327,28 @@ namespace KCDDB
 
     return true;
   }
+
+  QVariant CDInfo::get(const QString &type) const {
+    if(type == "id")
+      return id;
+    if(type == "artist")
+      return artist;
+    if(type == "title")
+      return title;
+    if(type == "genre")
+      return genre;
+    if(type == "category")
+      return category;
+    if(type == "extd")
+      return extd;
+    if(type == "year")
+      return year;
+    if(type == "length")
+      return length;
+    if(type == "revision")
+      return revision;
+    return QVariant();
+  }
 }
 
 // vim:tabstop=2:shiftwidth=2:expandtab:cinoptions=(s,U1,m1
