@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003-2004 Richard Lrkng <nouseforaname@home.se>
+  Copyright (C) 2003-2004 Richard Lärkäng <nouseforaname@home.se>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -19,8 +19,8 @@
 
 #include "syncsmtpsubmit.h"
 #include "cdinfo.h"
-#include <ktempfile.h>
 #include <kio/netaccess.h>
+#include <kio/job.h>
 
 namespace KCDDB
 {
@@ -36,32 +36,9 @@ namespace KCDDB
 
   }
 
-  CDDB::Result SyncSMTPSubmit::submit( const CDInfo& cdInfo, const TrackOffsetList& offsetList )
+  CDDB::Result SyncSMTPSubmit::runJob(KIO::Job* job)
   {
-    makeDiskData( cdInfo, offsetList );
-
-    if (!validCategory(cdInfo.category))
-      return InvalidCategory;
-
-    QString subject = QString("cddb %1 %2").arg(cdInfo.category, cdInfo.id);
-    makeURL(subject);
-
-    KTempFile tmp;
-    tmp.setAutoDelete(true);
-
-    QTextStream *textStream = tmp.textStream();
-
-    if (!textStream)
-      return UnknownError;
-
-    textStream->setEncoding(QTextStream::UnicodeUTF8);
-
-    *textStream << diskData_ << flush;
-
-    KURL tmpUrl;
-    tmpUrl.setPath( tmp.name() );
-
-    if ( KIO::NetAccess::file_copy( tmpUrl , url_ ) )
+    if ( KIO::NetAccess::synchronousRun(job, 0) )
       return Success;
 
     return UnknownError;
