@@ -13,7 +13,9 @@
 
 #include "mpegVideoStream.h"
 
+#include <iostream>
 
+using namespace std;
 
 MpegVideoStream::MpegVideoStream(InputStream* input) {
 
