@@ -19,11 +19,9 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
-#include <unistd.h>
 #include <stdio.h>
 
 
-#define _DUMP_YUV_AS_FRAME   1
 #define _DUMP_YUV_AS_STREAM  2
 
 
