@@ -30,6 +30,8 @@
 #include "player.moc"
 #include "stereobuttonaction.h"
 #include "stereobuttonaction.moc"
+#include "vequalizer.h"
+#include "vequalizer.moc"
 
 #endif
 
